#!/bin/sh
#: <<"#FETCH_WOODRPG"
echo Fetching WoodRPG...
if [ -f woodrpg.7z ]; then #use cache
	7z x -y woodrpg.7z
else
	svn co http://woodrpg.googlecode.com/svn/trunk/ woodrpg
	7z a -mx=9 -xr0!*/.svn/* -xr0!*/.svn woodrpg.7z woodrpg #cache. This cache is also meant to meet GPL condition...
fi

cp -f patch/libelm_Makefile_dldi woodrpg/libelm/Makefile
dos2unix woodrpg/libelm/source/ff.c
patch -p0 < patch/libelm_ff.patch

cp -f patch/libunds_dldi.c woodrpg/libunds/source/arm9/dldi.c
cp -f patch/libunds_dldi_stub.s woodrpg/libunds/source/arm9/dldi_stub.s

#cp -f patch/akloader_patches_pr.cpp woodrpg/akloader/arm9/source/patches_pr.cpp
#FETCH_WOODRPG

cd woodrpg

#: <<"#__BUILD_GUI"

cp -f ../patch/{dsrom.h,globalsettings.cpp,globalsettings.h,main.cpp,savemngr.cpp,romloader.cpp} akmenu4/arm9/source/
cp -f ../patch/makefile_all makefile
rm -f akmenu4/arm9/data/r4/*.bin
rm -f akmenu4/arm9/data/rpg/*.bin

#: <<"#BUILD_WOODR4"
echo Building WoodR4...
make >/dev/null
tar -C ../build/ -xzf akmenu_3in1.tar.gz
cp -a ../build/akmenu_3in1/__rpg ../build/
cp -f dldi/r4_sd/r4_sd.dldi ../build/__rpg/r4_sd.dldi
cp -f dldi/r4_sd/r4_sd.dldi ../build/__rpg/r4ds.dldi
rm -rf ../build/akmenu_3in1
cp -f akmenu4/akmenu4.nds ../build/woodrpg_mod.nds
cp -f akmenu4/akmenu4_r4.nds ../build/woodr4.nds
make clean >/dev/null
dldipatch ../build/__rpg/rpg_nand.dldi ../build/woodrpg_mod.nds
dldipatch ../patch/r4tf.dldi ../build/woodr4.nds
modifybanner ../build/woodr4.nds "Wood R4 mod;with autorunWithLastRom"
#BUILD_WOODR4

#: <<"#BUILD_WOODR4SDHC"
echo Building WoodR4sdhc...
cp -f ../patch/romloader_r4sdhc.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodr4sdhc.nds
make clean >/dev/null
dldipatch ../patch/r4tf.dldi ../build/woodr4sdhc.nds
modifybanner ../build/woodr4sdhc.nds "Wood R4 SDHC;partial clone support;sav defragment"
#BUILD_WOODR4SDHC

#: <<"#BUILD_WOODILS"
echo Building WoodiLS...
cp -f ../patch/romloader_ils.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodils.nds
make clean >/dev/null
dldipatch ../patch/ex4tf.dldi ../build/woodils.nds
modifybanner ../build/woodils.nds "Wood iLS;for R4iLS;only for SD <=4GB"
#BUILD_WOODILS

#: <<"#BUILD_WOODEX4"
echo Building WoodEX4...
cp -f ../patch/romloader_ex4.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodex4.nds
make clean >/dev/null
dldipatch ../patch/ex4tf.dldi ../build/woodex4.nds
modifybanner ../build/woodex4.nds "Wood EX4;for R4iLS/EX4DS;nds/sav defragment"
#BUILD_WOODEX4

#: <<"#BUILD_WOODM3"
echo Building WoodM3...
cp -f ../patch/romloader_m3.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodm3.nds
make clean >/dev/null
dldipatch ../patch/m3ds.dldi ../build/woodm3.nds
modifybanner ../build/woodm3.nds "Wood M3;for M3Real/M3iZero;nds/sav defragment"
#BUILD_WOODM3

: <<"#BUILD_WOODDSTT" #bad...
echo Building WoodDSTT...
cp -f ../patch/romloader_dstt.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/wooddstt.nds
make clean >/dev/null
dldipatch ../patch/ttio.dldi ../build/wooddstt.nds
modifybanner ../build/wooddstt.nds "Wood DSTT;for DSTT/SCDSONE(i);nds/sav defragment"
#BUILD_WOODDSTT

cp -f ../patch/fatx.h akmenu4/arm9/source/fatx.h

#: <<"#BUILD_WOODR4LS"
echo Building WoodR4LS...
cp -f ../patch/romloader_r4ls.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodr4ls.nds
make clean >/dev/null
dldipatch ../patch/r4tf.dldi ../build/woodr4ls.nds
modifybanner ../build/woodr4ls.nds "Wood R4LS;R4_AK_Special lives longer"
#BUILD_WOODR4LS

#: <<"#BUILD_WOODRPG_AK2i"
echo Building WoodRPG AK2i...
cp -f ../patch/romloader_ak2i.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/woodrpg_ak2i.nds
make clean >/dev/null
modifybanner ../build/woodrpg_ak2i.nds "Wood R4;modified for akloader"
#BUILD_WOODRPG_AK2i

cp -f ../patch/dldi.h akmenu4/arm9/source/dldi.h
cp -f ../patch/main_waio.cpp akmenu4/arm9/source/main.cpp

#: <<"#BUILD_WAIO"
echo Building WAIO...
cp -f ../patch/romloader_waio.cpp akmenu4/arm9/source/romloader.cpp
make akmenu4/_DS_MENU.DAT >/dev/null
cp -f akmenu4/akmenu4_r4.nds ../build/waio.nds
make clean >/dev/null
modifybanner ../build/waio.nds "WAIO - Wood All In One;another frontend for;MoonShell2 Extlink Wrapper"
#BUILD_WAIO

#__BUILD_GUI

#: <<"#__BUILD_LOADER"

cp -f ../patch/akloader_main_ils.cpp akloader/arm9/source/main.cpp
cp -f ../patch/dldi.c akloader/arm9/source/dldi.c
cp -f ../patch/akloader_Makefile_fix akloader/Makefile
cp -f ../patch/akloader_dbgtool.h akloader/arm9/source/dbgtool.h #dldi() uses dbg_printf()

#: <<"#BUILD_ILSLOADER"
echo Building ilsloader.nds...
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/ilsloader.nds
make clean >/dev/null
dldipatch ../patch/ex4tf.dldi ../build/__rpg/ilsloader.nds
#BUILD_ILSLOADER

cp -f ../patch/akloader_rpgmaps_sav.cpp akloader/arm9/source/rpgmaps.cpp

#: <<"#BUILD_R4LOADERSDHC"
echo Building r4loadersdhc.nds...
cp -f ../patch/r4sdhc/{save_nand,sd_save}.bin akloader/arm9/data/r4/
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/r4loadersdhc.nds
make clean >/dev/null
dldipatch ../patch/r4tf.dldi ../build/__rpg/r4loadersdhc.nds
#BUILD_R4LOADERSDHC

cp -f ../patch/akloader_main.cpp akloader/arm9/source/main.cpp #lock softreset (B4 command not supported)
cp -f ../patch/akloader_rpgmaps_nds.cpp akloader/arm9/source/rpgmaps.cpp

#: <<"#BUILD_EX4LOADER"
echo Building ex4loader.nds...
cp -f ../patch/ex4/{save_nand,sd_save,dma4}.bin akloader/arm9/data/r4/
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/ex4loader.nds
make clean >/dev/null
dldipatch ../patch/ex4tf.dldi ../build/__rpg/ex4loader.nds
#BUILD_EX4LOADER

#: <<"#BUILD_M3LOADER"
echo Building m3loader.nds...
cp -f ../patch/m3/{save_nand,sd_save,dma4}.bin akloader/arm9/data/r4/
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/m3loader.nds
make clean >/dev/null
dldipatch ../patch/m3ds.dldi ../build/__rpg/m3loader.nds
#BUILD_M3LOADER

: <<"#BUILD_DSTTLOADER" #bad...
echo Building dsttloader.nds...
cp -f ../patch/dstt/{save_nand,sd_save,dma4}.bin akloader/arm9/data/r4/
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/dsttloader.nds
make clean >/dev/null
dldipatch ../patch/ttio.dldi ../build/__rpg/dsttloader.nds
#BUILD_DSTTLOADER

cp -f ../patch/akloader_rpgmaps_nds_dsttsd.cpp akloader/arm9/source/rpgmaps.cpp

: <<"#BUILD_DSTTSDLOADER" #bad...
echo Building dsttsdloader.nds...
cp -f ../patch/dsttsd/{save_nand,sd_save,dma4}.bin akloader/arm9/data/r4/
make akloader/akloader_r4.nds >/dev/null
cp -f akloader/akloader_r4.nds ../build/__rpg/dsttsdloader.nds
make clean >/dev/null
dldipatch ../patch/ttio.dldi ../build/__rpg/dsttsdloader.nds
#BUILD_DSTTSDLOADER

#__BUILD_LOADER

cd ..
echo Done.
