/*
    globalsettings.cpp
    Copyright (C) 2007 Acekard, www.acekard.com
    Copyright (C) 2007-2009 somebody
    Copyright (C) 2009 yellow wood goblin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "globalsettings.h"
#include "inifile.h"
#include "systemfilenames.h"
#include "fontfactory.h"
#include <string.h>
#include <sys/stat.h>

cGlobalSettings::cGlobalSettings()
{
    fontHeight=12;
#if defined(_STORAGE_r4)
    brightness=1;
#endif
    language=1;
    langDirectory="lang_en";
    uiName="zelda";
    fileListType=0;
    romTrim=0;
    showHiddenFiles=true;
    enterLastDirWhenBoot=true;
    scrollSpeed = EScrollFast;
    showGbaRoms=true;
    viewMode=EViewIcon;
    gbaSleepHack=false;
    gbaAutoSave=false;
    Animation=true;
    cheats=false;
    softreset=false;
    dma=false;
    sdsave=false;
    cheatDB=false;
    slot2mode=ESlot2Ask;

    autorunWithLastRom      = false;
}

void cGlobalSettings::loadSettings()
{
    std::string temp;
    CIniFile ini( SFN_GLOBAL_SETTINGS );
    fontHeight = ini.GetInt( "system", "fontHeight", fontHeight );
    langDirectory = ini.GetString( "system", "langDirectory", langDirectory );
    uiName = ini.GetString( "system", "uiName", uiName );
    fileListType = ini.GetInt( "system", "fileListType", fileListType );
    romTrim = ini.GetInt( "system", "romTrim", romTrim );
    showHiddenFiles = ini.GetInt( "system", "showHiddenFiles", showHiddenFiles );
    enterLastDirWhenBoot = ini.GetInt( "system", "enterLastDirWhenBoot", enterLastDirWhenBoot );
    gbaSleepHack=ini.GetInt("system","gbaSleepHack",gbaSleepHack);
    gbaAutoSave=ini.GetInt("system","gbaAutoSave",gbaAutoSave);
    Animation=ini.GetInt("system","Animation",Animation);
    cheats=ini.GetInt("system","cheats",cheats);
    softreset=ini.GetInt("system","softreset",softreset);
    dma=ini.GetInt("system","dma",dma);
    sdsave=ini.GetInt("system","sdsave",sdsave);

    autorunWithLastRom = ini.GetInt( "system", "autorunWithLastRom", autorunWithLastRom );

    temp = ini.GetString( "system", "scrollSpeed", "fast" );
    if(strcmp(temp.c_str(),"slow") == 0)
        scrollSpeed = EScrollSlow;
    else if(strcmp(temp.c_str(),"medium") == 0)
        scrollSpeed = EScrollMedium;
    else if(strcmp(temp.c_str(),"fast")==0)
        scrollSpeed = EScrollFast;

    temp = ini.GetString( "system", "viewMode", "icon" );
    if(strcmp(temp.c_str(),"list") == 0)
        viewMode = EViewList;
    else if(strcmp(temp.c_str(),"icon") == 0)
        viewMode = EViewIcon;
    else if(strcmp(temp.c_str(),"internal") == 0)
        viewMode = EViewInternal;

    temp = ini.GetString( "system", "slot2mode", "ask" );
    if(strcmp(temp.c_str(),"ask") == 0)
        slot2mode = ESlot2Ask;
    else if(strcmp(temp.c_str(),"gba") == 0)
        slot2mode = ESlot2Gba;
    else if(strcmp(temp.c_str(),"nds") == 0)
        slot2mode = ESlot2Nds;

    struct stat st;
    if(0==stat(SFN_CHEATS,&st)) cheatDB=true;

#if defined(_STORAGE_r4)
    CIniFile iniBacklight(SFN_BACKLIGHT);
    brightness=iniBacklight.GetInt("brightness","brightness",brightness);
    setBrightness(brightness);
#endif
}

void cGlobalSettings::saveSettings()
{
    // the commented code means those parameters are not allow to change in menu
    CIniFile ini( SFN_GLOBAL_SETTINGS );
    //ini.SetInt( "system", "fontHeight", fontHeight );
    ini.SetString( "system", "uiName", uiName );
    ini.SetString( "system", "langDirectory", langDirectory );
    ini.SetInt( "system", "fileListType", fileListType );
    ini.SetInt( "system", "romTrim", romTrim );
    ini.SetInt( "system", "showHiddenFiles", showHiddenFiles );
    ini.SetInt("system","gbaSleepHack",gbaSleepHack);
    ini.SetInt("system","gbaAutoSave",gbaAutoSave);
    ini.SetInt("system","Animation",Animation);
    ini.SetInt("system","cheats",cheats);
    ini.SetInt("system","softreset",softreset);
    ini.SetInt("system","dma",dma);
    ini.SetInt("system","sdsave",sdsave);

    if(scrollSpeed==EScrollSlow) ini.SetString( "system", "scrollSpeed", "slow" );
    else if(scrollSpeed==EScrollMedium) ini.SetString( "system", "scrollSpeed", "medium" );
    else if(scrollSpeed==EScrollFast) ini.SetString( "system", "scrollSpeed", "fast" );

    if(viewMode==EViewList) ini.SetString( "system", "viewMode", "list" );
    else if(viewMode==EViewIcon) ini.SetString( "system", "viewMode", "icon" );
    else if(viewMode==EViewInternal) ini.SetString( "system", "viewMode", "internal" );

    switch(slot2mode)
    {
      case ESlot2Gba:
        ini.SetString("system","slot2mode","gba");
        break;
      case ESlot2Nds:
        ini.SetString("system","slot2mode","nds");
        break;
      default:
        ini.SetString("system","slot2mode","ask");
        break;
    }

    ini.SaveIniFile( SFN_GLOBAL_SETTINGS );
}

u32 cGlobalSettings::CopyBufferSize(void)
{
  if(font().FontRAM()<300*1024) return 1024*1024;
  return 512*1024;
}

void cGlobalSettings::nextBrightness(void)
{
#if defined(_STORAGE_rpg) || defined(_STORAGE_ak2i)
  fifoSendValue32(FIFO_USER_01,MENU_MSG_BRIGHTNESS_NEXT);
#elif defined(_STORAGE_r4)
  fifoSendValue32(FIFO_USER_01,MENU_MSG_BRIGHTNESS_GET);
  while(!fifoCheckValue32(FIFO_USER_01));
  u32 currentLevel=fifoGetValue32(FIFO_USER_01);
  brightness=(currentLevel+1)&3;

  setBrightness(brightness);
  CIniFile ini(SFN_BACKLIGHT);
  ini.SetInt("brightness","brightness",brightness);
  ini.SaveIniFile(SFN_BACKLIGHT);
#endif
}

#if defined(_STORAGE_r4)
void cGlobalSettings::setBrightness(u32 level)
{
  fifoSendValue32(FIFO_USER_01,MENU_MSG_BRIGHTNESS_SET0+(brightness&3));
}
#endif
